@@ -4,13 +4,32 @@
 
 /* General project wide configuration */
 
+/*
+ * Author: Graeme W. Gill
+ *
+ * Copyright 2006 - 2016, Graeme W. Gill
+ * All rights reserved.
+ *
+ * This material is licenced under the GNU GENERAL PUBLIC LICENSE Version 2 or later :-
+ * see the License2.txt file for licencing details.
+ */
+
+#ifdef __cplusplus
+	extern "C" {
+#endif
+
 /* Version of Argyll release */
 /* Bug fix = 4 bits */
 /* minor number = 8 bits */
 /* major number = 8 bits */
 
-#define ARGYLL_VERSION 0x01083
-#define ARGYLL_VERSION_STR "1.8.3"
+#ifndef USE_NG_VERSION 
+# define ARGYLL_VERSION 0x01092
+# define ARGYLL_VERSION_STR "1.9.2"
+#else
+# define ARGYLL_VERSION NG_VERSION
+# define ARGYLL_VERSION_STR "NG_VERSION_STR"
+#endif
 
 #if defined(NT)
 # if defined(_WIN64)
@@ -21,13 +40,13 @@
 #endif
 #if defined(UNIX)
 # if defined(__APPLE__)
-#  if defined(__LP64__)
+#  if defined(__P64__)
 #   define ARGYLL_BUILD_STR "OS X 64 bit" 
 #  else
 #   define ARGYLL_BUILD_STR "OS X 32 bit" 
 #  endif
 # else
-#  if defined(__LP64__)
+#  if defined(__P64__)
 #   define ARGYLL_BUILD_STR "Linux 64 bit" 
 #  else
 #   define ARGYLL_BUILD_STR "Linux 32 bit" 
@@ -38,15 +57,26 @@
 /* Maximum file path length */
 #define MAXNAMEL 1024
 
+/* Maximum number of entries to setup for calibration */
+#define MAX_CAL_ENT 16384
+
 /* A simpler #define to remove __APPLE__ from non OS X code */
-#if defined(UNIX) && !defined(__APPLE__)
-# define UNIX_X11       /* Unix like using X11 */
-#else
-# undef UNIX_X11
+#if defined(UNIX)
+# if defined(__APPLE__)
+#  define UNIX_APPLE
+#  undef  UNIX_X11
+# else
+#  define UNIX_X11
+#  undef  UNIX_APPLE
+# endif
 #endif
 
-#if defined(UNIX) && !defined(__APPLE__)
-#define USE_UCMM		/* Enable the Unix micro CMM */
+#ifdef UNIX_X11
+# define USE_UCMM		/* Enable the Unix micro CMM */
+#endif
+
+#ifdef __cplusplus
+	}
 #endif
 
 #endif /* __CONFIG_H__ */
