@@ -177,6 +177,9 @@ struct _munkiimp {
 
 	double intclkp;				/* Integration clock period (computed from tickdur) */
 
+	xcalstd native_calstd;		/* Instrument native calibration standard */
+	xcalstd target_calstd;		/* Returned calibration standard */
+
 	/* Current state of hardware (~~99 are all these used ??) */
 	double c_inttime;			/* Integration period (=inttime + deadtime) */
 	int c_measmodeflags;		/* Measurement mode flags (set by trigger() */
@@ -418,6 +421,7 @@ munki_code munki_imp_calibrate(
 munki *p,
 inst_cal_type *calt,	/* Calibration type to do/remaining */
 inst_cal_cond *calc,	/* Current condition/desired condition */
+inst_calc_id_type *idtype,	/* Condition identifier type */
 char id[100]			/* Condition identifier (ie. white reference ID) */
 );
 
